package sun.security.util;

import java.security.cert.CertPath;
import java.util.Base64;
import java.security.GeneralSecurityException;
import java.util.Iterator;
import sun.security.pkcs.SignerInfo;
import java.util.jar.Attributes;
import java.io.ByteArrayInputStream;
import java.util.jar.Manifest;
import java.util.jar.JarException;
import java.util.List;
import java.util.Hashtable;
import java.security.SignatureException;
import java.security.NoSuchAlgorithmException;
import java.security.cert.CertificateException;
import java.io.IOException;
import java.util.Locale;
import sun.security.jca.Providers;
import java.security.Timestamp;
import java.util.Map;
import java.security.cert.CertificateFactory;
import java.security.MessageDigest;
import java.util.HashMap;
import sun.security.pkcs.PKCS7;
import java.security.CodeSigner;
import java.util.ArrayList;

public class SignatureFileVerifier
{
    private static final Debug debug;
    private static final DisabledAlgorithmConstraints JAR_DISABLED_CHECK;
    private ArrayList<CodeSigner[]> signerCache;
    private static final String ATTR_DIGEST;
    private PKCS7 block;
    private byte[] sfBytes;
    private String name;
    private ManifestDigester md;
    private HashMap<String, MessageDigest> createdDigests;
    private boolean workaround;
    private CertificateFactory certificateFactory;
    private Map<String, Boolean> permittedAlgs;
    private Timestamp timestamp;
    private static final char[] hexc;
    
    public SignatureFileVerifier(ArrayList<CodeSigner[]> signerCache, ManifestDigester md, String s, byte[] bytes) throws IOException, CertificateException {
        this.workaround = false;
        this.certificateFactory = null;
        this.permittedAlgs = new HashMap();
        this.timestamp = null;
        Object startJarVerification = null;
        try {
            startJarVerification = Providers.startJarVerification();
            this.block = new PKCS7(bytes);
            this.sfBytes = this.block.getContentInfo().getData();
            this.certificateFactory = CertificateFactory.getInstance("X509");
        }
        finally {
            Providers.stopJarVerification(startJarVerification);
        }
        this.name = s.substring(0, s.lastIndexOf(46)).toUpperCase(Locale.ENGLISH);
        this.md = md;
        this.signerCache = signerCache;
    }
    
    public boolean needSignatureFileBytes() {
        return this.sfBytes == null;
    }
    
    public boolean needSignatureFile(String anotherString) {
        return this.name.equalsIgnoreCase(anotherString);
    }
    
    public void setSignatureFile(byte[] sfBytes) {
        this.sfBytes = sfBytes;
    }
    
    public static boolean isBlockOrSF(String s) {
        return s.endsWith(".SF") || s.endsWith(".DSA") || s.endsWith(".RSA") || s.endsWith(".EC");
    }
    
    public static boolean isSigningRelated(String s) {
        s = s.toUpperCase(Locale.ENGLISH);
        if (!s.startsWith("META-INF/")) {
            return false;
        }
        s = s.substring(9);
        if (s.indexOf(47) != -1) {
            return false;
        }
        if (isBlockOrSF(s) || s.equals("MANIFEST.MF")) {
            return true;
        }
        if (s.startsWith("SIG-")) {
            int lastIndex = s.lastIndexOf(46);
            if (lastIndex != -1) {
                String substring = s.substring(lastIndex + 1);
                if (substring.length() > 3 || substring.length() < 1) {
                    return false;
                }
                int i = 0;
                while (i < substring.length()) {
                    int char1 = substring.charAt(i);
                    if ((char1 < 'A' || char1 > 'Z') && (char1 < '0' || char1 > '9')) {
                        return false;
                    }
                    ++i;
                }
            }
            return true;
        }
        else {
            return false;
        }
    }
    
    private MessageDigest getDigest(String key) throws SignatureException {
        if (this.createdDigests == null) {
            this.createdDigests = new HashMap();
        }
        MessageDigest instance = (MessageDigest)this.createdDigests.get(key);
        if (instance == null) {
            try {
                instance = MessageDigest.getInstance(key);
                this.createdDigests.put(key, instance);
            }
            catch (NoSuchAlgorithmException ex) {}
        }
        return instance;
    }
    
    public void process(Hashtable<String, CodeSigner[]> hashtable, List<Object> list) throws IOException, SignatureException, NoSuchAlgorithmException, JarException, CertificateException {
        Object startJarVerification = null;
        try {
            startJarVerification = Providers.startJarVerification();
            this.processImpl(hashtable, list);
        }
        finally {
            Providers.stopJarVerification(startJarVerification);
        }
    }
    
    private void processImpl(Hashtable<String, CodeSigner[]> hashtable, List<Object> list) throws IOException, SignatureException, NoSuchAlgorithmException, JarException, CertificateException {
        Manifest manifest = new Manifest();
        manifest.read(new ByteArrayInputStream(this.sfBytes));
        String value = manifest.getMainAttributes().getValue(Attributes.Name.SIGNATURE_VERSION);
        if (value == null || !value.equalsIgnoreCase("1.0")) {
            return;
        }
        SignerInfo[] verify = this.block.verify(this.sfBytes);
        if (verify == null) {
            throw new SecurityException("cannot verify signature block file " + this.name);
        }
        CodeSigner[] signers = this.getSigners(verify, this.block);
        if (signers == null) {
            return;
        }
        int length = signers.length;
        int i = 0;
        while (i < length) {
            CodeSigner codeSigner = signers[i];
            if (SignatureFileVerifier.debug != null) {
                SignatureFileVerifier.debug.println("Gathering timestamp for:  " + codeSigner.toString());
            }
            if (codeSigner.getTimestamp() == null) {
                this.timestamp = null;
                break;
            }
            if (this.timestamp == null) {
                this.timestamp = codeSigner.getTimestamp();
            }
            else if (this.timestamp.getTimestamp().before(codeSigner.getTimestamp().getTimestamp())) {
                this.timestamp = codeSigner.getTimestamp();
            }
            ++i;
        }
        Iterator<Map.Entry<String, Attributes>> iterator = manifest.getEntries().entrySet().iterator();
        int verifyManifestHash = this.verifyManifestHash(manifest, this.md, list);
        if (verifyManifestHash == 0 && !this.verifyManifestMainAttrs(manifest, this.md)) {
            throw new SecurityException("Invalid signature file digest for Manifest main attributes");
        }
        while (iterator.hasNext()) {
            Map.Entry<Object, Object> entry = (Map.Entry)iterator.next();
            String s = (String)entry.getKey();
            if (verifyManifestHash || this.verifySection((Attributes)entry.getValue(), s, this.md)) {
                if (s.startsWith("./")) {
                    s = s.substring(2);
                }
                if (s.startsWith("/")) {
                    s = s.substring(1);
                }
                this.updateSigners(signers, hashtable, s);
                if (SignatureFileVerifier.debug != null) {
                    SignatureFileVerifier.debug.println("processSignature signed name = " + s);
                }
                else {
                    continue;
                }
            }
            else if (SignatureFileVerifier.debug != null) {
                SignatureFileVerifier.debug.println("processSignature unsigned name = " + s);
            }
            else {
                continue;
            }
        }
        this.updateSigners(signers, hashtable, "META-INF/MANIFEST.MF");
        return;
    }
    
    boolean permittedCheck(String s, String str) {
        Boolean b = (Boolean)this.permittedAlgs.get(str);
        if (b == null) {
            try {
                SignatureFileVerifier.JAR_DISABLED_CHECK.permits(str, new ConstraintsParameters(this.timestamp));
            }
            catch (GeneralSecurityException ex) {
                this.permittedAlgs.put(str, Boolean.FALSE);
                this.permittedAlgs.put(s.toUpperCase(), Boolean.FALSE);
                if (SignatureFileVerifier.debug != null) {
                    if (ex.getMessage() != null) {
                        SignatureFileVerifier.debug.println(s + ":  " + ex.getMessage());
                    }
                    else {
                        SignatureFileVerifier.debug.println("Debug info only. " + s + ":  " + str + " was disabled, no exception msg given.");
                        ex.printStackTrace();
                    }
                }
                return false;
            }
            this.permittedAlgs.put(str, Boolean.TRUE);
            return true;
        }
        return (boolean)b;
    }
    
    String getWeakAlgorithms(String s) {
        String str = "";
        try {
            for (String s2 : this.permittedAlgs.keySet()) {
                if (s2.endsWith(s)) {
                    str = str + s2.substring(0, s2.length() - s.length()) + " ";
                }
                else {
                    continue;
                }
            }
        }
        catch (RuntimeException ex) {
            str = "Unknown Algorithm(s).  Error processing " + s + ".  " + ex.getMessage();
        }
        if (str.length() == 0) {
            return "Unknown Algorithm(s)";
        }
        return str;
    }
    
    private boolean verifyManifestHash(Manifest manifest, ManifestDigester manifestDigester, List<Object> list) throws IOException, SignatureException {
        Attributes mainAttributes = manifest.getMainAttributes();
        boolean n = false;
        boolean n2 = true;
        boolean n3 = false;
        for (Map.Entry entry : mainAttributes.entrySet()) {
            String string = entry.getKey().toString();
            if (string.toUpperCase(Locale.ENGLISH).endsWith("-DIGEST-MANIFEST")) {
                String substring = string.substring(0, string.length() - 16);
                n3 = true;
                if (!this.permittedCheck(string, substring)) {
                    continue;
                }
                n2 = false;
                list.add(string);
                list.add(entry.getValue());
                MessageDigest digest = this.getDigest(substring);
                if (digest != null) {
                    byte[] manifestDigest = manifestDigester.manifestDigest(digest);
                    byte[] decode = Base64.getMimeDecoder().decode((String)entry.getValue());
                    if (SignatureFileVerifier.debug != null) {
                        SignatureFileVerifier.debug.println("Signature File: Manifest digest " + substring);
                        SignatureFileVerifier.debug.println("  sigfile  " + toHex(decode));
                        SignatureFileVerifier.debug.println("  computed " + toHex(manifestDigest));
                        SignatureFileVerifier.debug.println();
                    }
                    if (MessageDigest.isEqual(manifestDigest, decode)) {
                        n = true;
                    }
                    else {
                        continue;
                    }
                }
                else {
                    continue;
                }
            }
            else {
                continue;
            }
        }
        if (SignatureFileVerifier.debug != null) {
            SignatureFileVerifier.debug.println("PermittedAlgs mapping: ");
            for (String str : this.permittedAlgs.keySet()) {
                SignatureFileVerifier.debug.println(str + " : " + ((Boolean)this.permittedAlgs.get(str)).toString());
            }
        }
        if (n3 && n2) {
            throw new SignatureException("Manifest hash check failed (DIGEST-MANIFEST). Disabled algorithm(s) used: " + this.getWeakAlgorithms("-DIGEST-MANIFEST"));
        }
        return n;
    }
    
    private boolean verifyManifestMainAttrs(Manifest manifest, ManifestDigester manifestDigester) throws IOException, SignatureException {
        Attributes mainAttributes = manifest.getMainAttributes();
        boolean n = true;
        boolean n2 = true;
        boolean n3 = false;
        for (Map.Entry entry : mainAttributes.entrySet()) {
            String string = entry.getKey().toString();
            if (string.toUpperCase(Locale.ENGLISH).endsWith(SignatureFileVerifier.ATTR_DIGEST)) {
                String substring = string.substring(0, string.length() - SignatureFileVerifier.ATTR_DIGEST.length());
                n3 = true;
                if (!this.permittedCheck(string, substring)) {
                    continue;
                }
                n2 = false;
                MessageDigest digest = this.getDigest(substring);
                if (digest != null) {
                    byte[] digest2 = manifestDigester.get("Manifest-Main-Attributes", false).digest(digest);
                    byte[] decode = Base64.getMimeDecoder().decode((String)entry.getValue());
                    if (SignatureFileVerifier.debug != null) {
                        SignatureFileVerifier.debug.println("Signature File: Manifest Main Attributes digest " + digest.getAlgorithm());
                        SignatureFileVerifier.debug.println("  sigfile  " + toHex(decode));
                        SignatureFileVerifier.debug.println("  computed " + toHex(digest2));
                        SignatureFileVerifier.debug.println();
                    }
                    if (MessageDigest.isEqual(digest2, decode)) {
                        continue;
                    }
                    n = false;
                    if (SignatureFileVerifier.debug != null) {
                        SignatureFileVerifier.debug.println("Verification of Manifest main attributes failed");
                        SignatureFileVerifier.debug.println();
                        break;
                    }
                    break;
                }
                else {
                    continue;
                }
            }
            else {
                continue;
            }
        }
        if (SignatureFileVerifier.debug != null) {
            SignatureFileVerifier.debug.println("PermittedAlgs mapping: ");
            for (String str : this.permittedAlgs.keySet()) {
                SignatureFileVerifier.debug.println(str + " : " + ((Boolean)this.permittedAlgs.get(str)).toString());
            }
        }
        if (n3 && n2) {
            throw new SignatureException("Manifest Main Attribute check failed (" + SignatureFileVerifier.ATTR_DIGEST + ").  Disabled algorithm(s) used: " + this.getWeakAlgorithms(SignatureFileVerifier.ATTR_DIGEST));
        }
        return n;
    }
    
    private boolean verifySection(Attributes attributes, String str, ManifestDigester manifestDigester) throws IOException, SignatureException {
        boolean n = false;
        ManifestDigester.Entry value = manifestDigester.get(str, this.block.isOldStyle());
        boolean n2 = true;
        boolean n3 = false;
        if (value == null) {
            throw new SecurityException("no manifest section for signature file entry " + str);
        }
        if (attributes != null) {
            for (Map.Entry entry : attributes.entrySet()) {
                String string = entry.getKey().toString();
                if (string.toUpperCase(Locale.ENGLISH).endsWith("-DIGEST")) {
                    String substring = string.substring(0, string.length() - 7);
                    n3 = true;
                    if (!this.permittedCheck(string, substring)) {
                        continue;
                    }
                    n2 = false;
                    MessageDigest digest = this.getDigest(substring);
                    if (digest != null) {
                        boolean n4 = false;
                        byte[] decode = Base64.getMimeDecoder().decode((String)entry.getValue());
                        byte[] digesta;
                        if (this.workaround) {
                            digesta = value.digestWorkaround(digest);
                        }
                        else {
                            digesta = value.digest(digest);
                        }
                        if (SignatureFileVerifier.debug != null) {
                            SignatureFileVerifier.debug.println("Signature Block File: " + str + " digest=" + digest.getAlgorithm());
                            SignatureFileVerifier.debug.println("  expected " + toHex(decode));
                            SignatureFileVerifier.debug.println("  computed " + toHex(digesta));
                            SignatureFileVerifier.debug.println();
                        }
                        if (MessageDigest.isEqual(digesta, decode)) {
                            n = true;
                            n4 = true;
                        }
                        else if (!this.workaround) {
                            byte[] digestWorkaround = value.digestWorkaround(digest);
                            if (MessageDigest.isEqual(digestWorkaround, decode)) {
                                if (SignatureFileVerifier.debug != null) {
                                    SignatureFileVerifier.debug.println("  re-computed " + toHex(digestWorkaround));
                                    SignatureFileVerifier.debug.println();
                                }
                                this.workaround = true;
                                n = true;
                                n4 = true;
                            }
                        }
                        if (n4 == 0) {
                            throw new SecurityException("invalid " + digest.getAlgorithm() + " signature file digest for " + str);
                        }
                        continue;
                    }
                    else {
                        continue;
                    }
                }
                else {
                    continue;
                }
            }
        }
        if (SignatureFileVerifier.debug != null) {
            SignatureFileVerifier.debug.println("PermittedAlgs mapping: ");
            for (String str2 : this.permittedAlgs.keySet()) {
                SignatureFileVerifier.debug.println(str2 + " : " + ((Boolean)this.permittedAlgs.get(str2)).toString());
            }
        }
        if (n3 && n2) {
            throw new SignatureException("Manifest Main Attribute check failed (DIGEST).  Disabled algorithm(s) used: " + this.getWeakAlgorithms("DIGEST"));
        }
        return n;
    }
    
    private CodeSigner[] getSigners(SignerInfo[] array, PKCS7 block) throws IOException, NoSuchAlgorithmException, SignatureException, CertificateException {
        Object o = null;
        for (int i = 0; i < array.length; ++i) {
            SignerInfo signerInfo = array[i];
            ArrayList<X509Certificate> certificateChain = signerInfo.getCertificateChain(block);
            CertPath generateCertPath = this.certificateFactory.generateCertPath(certificateChain);
            if (o == null) {
                o = new ArrayList();
            }
            o.add(new CodeSigner(generateCertPath, signerInfo.getTimestamp()));
            if (SignatureFileVerifier.debug != null) {
                SignatureFileVerifier.debug.println("Signature Block Certificate: " + certificateChain.get(0));
            }
        }
        if (o != null) {
            return (CodeSigner[])o.toArray(new CodeSigner[o.size()]);
        }
        return null;
    }
    
    static String toHex(byte[] array) {
        StringBuilder sb = new StringBuilder(array.length * 2);
        for (int i = 0; i < array.length; ++i) {
            sb.append(SignatureFileVerifier.hexc[array[i] >> 4 & 0xF]);
            sb.append(SignatureFileVerifier.hexc[array[i] & 0xF]);
        }
        return sb.toString();
    }
    
    static boolean contains(CodeSigner[] array, CodeSigner obj) {
        int i = 0;
        while (i < array.length) {
            if (array[i].equals(obj)) {
                return true;
            }
            ++i;
        }
        return false;
    }
    
    static boolean isSubSet(CodeSigner[] array, CodeSigner[] array2) {
        if (array2 == array) {
            return true;
        }
        int i = 0;
        while (i < array.length) {
            if (!contains(array2, array[i])) {
                return false;
            }
            ++i;
        }
        return true;
    }
    
    static boolean matches(CodeSigner[] array, CodeSigner[] array2, CodeSigner[] array3) {
        if (array2 == null && array == array3) {
            return true;
        }
        if (array2 != null && !isSubSet(array2, array)) {
            return false;
        }
        if (!isSubSet(array3, array)) {
            return false;
        }
        int i = 0;
        while (i < array.length) {
            if ((array2 == null || !contains(array2, array[i])) && !contains(array3, array[i])) {
                return false;
            }
            ++i;
        }
        return true;
    }
    
    void updateSigners(CodeSigner[] array, Hashtable<String, CodeSigner[]> hashtable, String key) {
        CodeSigner[] array2 = (CodeSigner[])hashtable.get(key);
        int i = this.signerCache.size() - 1;
        while (i != -1) {
            CodeSigner[] value = (CodeSigner[])this.signerCache.get(i);
            if (matches(value, array2, array)) {
                hashtable.put(key, value);
                return;
            }
            --i;
        }
        CodeSigner[] array3;
        if (array2 == null) {
            array3 = array;
        }
        else {
            array3 = new CodeSigner[array2.length + array.length];
            System.arraycopy(array2, 0, array3, 0, array2.length);
            System.arraycopy(array, 0, array3, array2.length, array.length);
        }
        this.signerCache.add(array3);
        hashtable.put(key, array3);
    }
    
    static {
        debug = Debug.getInstance("jar");
        JAR_DISABLED_CHECK = new DisabledAlgorithmConstraints("jdk.jar.disabledAlgorithms");
        ATTR_DIGEST = "-DIGEST-Manifest-Main-Attributes".toUpperCase(Locale.ENGLISH);
        hexc = new char[] { '0', '1', '2', '3', '4', '5', '6', '7', '8', '9', 'a', 'b', 'c', 'd', 'e', 'f' };
    }
}
